//+------------------------------------------------------------------+
//|                   FSB__Bar_Opening - Bar_Closing.mq4 v0.0.2 Beta |
//|                                 Copyright  2009, Miroslav Popov |
//|                                              http://forexsb.com/ |
//|                                                                  |
//|         Free for use and modification                            |
//|                                                                  |
//| An exmple EA pattern:                                            |
//| * Enter the market at Bar Opening                                |
//| * Exit the market at Bar Closing                                 |
//+------------------------------------------------------------------+
#property copyright "Copyright  2009, Miroslav Popov"
#property link      "http://forexsb.com/"

// The time span before bar closing time in seconds.
// It determines the time interval for position closing.
extern double ClosingTimeSpan = 10;



bool bIsFirstTick;// It is used to catch the Bar Opening.
datetime dtCurrentBarOpeningTime;

int init()
{
   bIsFirstTick = false;
   dtCurrentBarOpeningTime = Time[0];
   return(0);
}

int start()
{
   // Is this the first tick for the bar
   bIsFirstTick = (dtCurrentBarOpeningTime != Time[0]);
   if(bIsFirstTick)
   {
      dtCurrentBarOpeningTime = Time[0];
   }
   
   // Check if there are open positions from the previous bar
   int iOrders = OrdersTotal();
   if(iOrders > 0)
      ClosePositionsAtBarClosing(true, true, ClosingTimeSpan);
 
 
   
   // Opening Logic Conditions
   bool bLongEntryAllowed  = false;
   bool bShortEntryAllowed = false;

   // Put the entry logic rules here ...
   
   if(iMA(NULL, 0, 13, 0, MODE_SMA, PRICE_CLOSE, 1) >
      iMA(NULL, 0, 13, 0, MODE_SMA, PRICE_CLOSE, 2))
      bLongEntryAllowed = true;
   else
      bShortEntryAllowed = true;


   // Entry at Bar Opening
   iOrders = OrdersTotal();
   if(iOrders == 0)
   {
      if(bLongEntryAllowed || bShortEntryAllowed)
         OpenPositionAtBarOpening(bLongEntryAllowed, bShortEntryAllowed);
         
      iOrders = OrdersTotal();
      if(iOrders > 0)
         return(0);
   }



   // Exit Logic Conditions
   bool bCloseLong  = true;
   bool bCloseShort = true;

   // Put the exit logic rules here ...
   
   // Exit
   if(bCloseLong || bCloseShort)
      ClosePositionsAtBarClosing(bCloseLong, bCloseShort, ClosingTimeSpan);
      
      
   // Debug
   PrintLastError();
}

///
/// Entry at a Bar Opening price.
///
/// MetaTrader does not provide an onBarOpen event so we check the current tick volume.
/// We open a position when the current tick volume is equal to 1.
///
int OpenPositionAtBarOpening(bool bLongEntryAllowed, bool bShortEntryAllowed)
{
   if(!bLongEntryAllowed && !bShortEntryAllowed)
   { // An entry is not allowed.
      return(0);
   } 
   
   // Check for Bar Opening
   if(!bIsFirstTick)
   {  // This is not the first tick.
      return(0);
   } 
   
   int iLots = 1;

   // Check the free margin.
   if(AccountFreeMargin() < (1000 * iLots))
   {
      Print("We do not have money enough! Free Margin = ", AccountFreeMargin());
      return(0);
   }
   
   int ticket = 0;
   if(bLongEntryAllowed)
   {
      ticket = OrderSend(Symbol(), OP_BUY, iLots, Ask, 3, 0, 0, "Bar Opening", 0 ,0 , Green);
   }
   if(bShortEntryAllowed)
   {
      ticket = OrderSend(Symbol(), OP_SELL, iLots, Bid, 3, 0, 0, "Bar Opening", 0 ,0 , Red);
   }
   
   if(ticket > 0)
   {
      if(OrderSelect(ticket, SELECT_BY_TICKET, MODE_TRADES))
         Print("Position opened at: ", OrderOpenPrice());
   }
   else 
   {
      Print("Error opening a position: ", GetErrorDescription(GetLastError()));
   }
      
   return(0);
}


///
/// Exit at a Bar Closing price (almost).
///
/// MetaTrader does not provide an onBarClose event so we are not able to close a position
/// exactly at Bar Closing. We workaround the problem by closing the position within a time span
/// near to the bar closing time. In the cases when there is no any ticks within this period,
/// we close the position att he next bar opening price.
///
int ClosePositionsAtBarClosing(bool bCloseLong, bool bCloseShort, datetime dtClosingTimeSpan)
{
   int  iOrders = OrdersTotal();
   bool bIsOpen = false;

   for(int iOrder = 0; iOrder < iOrders; iOrder++)
   {
      OrderSelect(iOrder, SELECT_BY_POS, MODE_TRADES);
      
      if((OrderType() == OP_BUY || OrderType() == OP_SELL) && OrderSymbol() == Symbol())
      {  // There is an open position for this symbol.

         datetime dtOpeningTime     = Time[0] - TimeSeconds(Time[0]); // The opening time of current bar
         datetime dtClosingTime     = dtOpeningTime + Period() * 60;  // The closing time of current bars
         datetime dtCurrentTickTime = TimeCurrent() ;                 // The time of current tick
         
         if(dtCurrentTickTime > dtClosingTime - dtClosingTimeSpan ||  bIsFirstTick)
         {  // The current tick is within the closing time span or this is the first tick of the bar.
            if(OrderType() == OP_BUY && bCloseLong)
            {  // Close a long position
               bIsOpen = OrderClose(OrderTicket(), OrderLots(), Bid, 3, Violet);
               
               if(bIsOpen)
                  Print("Long position closed at: ", OrderClosePrice());
               else
                  Print("Error closing a position: ", GetErrorDescription(GetLastError()));
            }
            else if(OrderType() == OP_SELL && bCloseShort)
            {  // Close a short position
               bIsOpen = OrderClose(OrderTicket(), OrderLots(), Ask, 3, Violet);
               
               if(bIsOpen)
                  Print("Short position closed at: ", OrderClosePrice());
               else
                  Print("Error closing a position: ", GetErrorDescription(GetLastError()));
            }
         }
      }
   }
   
   return(0);
}



///
/// Prints the last error, if any.
///
void PrintLastError()
{
   int iLastError = GetLastError();
   
   if(iLastError != 0 && iLastError != 1)
      Print("Error: ", GetErrorDescription(iLastError));
}

///
/// Gets the error description
///
string GetErrorDescription(int iLastError)
{
	string sErrorDescription = "Unknown error";

	switch(iLastError)
	{
		case 0:    sErrorDescription = "No value";                                                 break;
		case 1:    sErrorDescription = "No error";                                                 break;
		case 2:    sErrorDescription = "Common error";                                             break;
		case 3:    sErrorDescription = "Invalid trade parameters";                                 break;
		case 4:    sErrorDescription = "Trade server is busy";                                     break;
		case 5:    sErrorDescription = "Old version of the client terminal";                       break;
		case 6:    sErrorDescription = "No connection with trade server";                          break;
		case 7:    sErrorDescription = "Not enough rights";                                        break;
		case 8:    sErrorDescription = "Too frequent requests";                                    break;
		case 9:    sErrorDescription = "Malfunctional trade operation (never returned error)";     break;
		case 64:   sErrorDescription = "Account disabled";                                         break;
		case 65:   sErrorDescription = "Invalid account";                                          break;
		case 128:  sErrorDescription = "Trade timeout";                                            break;
		case 129:  sErrorDescription = "Invalid price";                                            break;
		case 130:  sErrorDescription = "Invalid stops";                                            break;
		case 131:  sErrorDescription = "Invalid trade volume";                                     break;
		case 132:  sErrorDescription = "Market is closed";                                         break;
		case 133:  sErrorDescription = "Trade is disabled";                                        break;
		case 134:  sErrorDescription = "Not enough money";                                         break;
		case 135:  sErrorDescription = "Price changed";                                            break;
		case 136:  sErrorDescription = "Off quotes";                                               break;
		case 137:  sErrorDescription = "Broker is busy (never returned error)";                    break;
		case 138:  sErrorDescription = "Requote";                                                  break;
		case 139:  sErrorDescription = "Order is locked";                                          break;
		case 140:  sErrorDescription = "Long positions only allowed";                              break;
		case 141:  sErrorDescription = "Too many requests";                                        break;
		case 145:  sErrorDescription = "Modification denied because order too close to market";    break;
		case 146:  sErrorDescription = "Trade context is busy";                                    break;
		case 147:  sErrorDescription = "Expirations are denied by broker";                         break;
		case 148:  sErrorDescription = "Amount of open and pending orders has reached the limit";  break;
		case 4000: sErrorDescription = "No error (never generated code)";                          break;
		case 4001: sErrorDescription = "Wrong function pointer";                                   break;
		case 4002: sErrorDescription = "Array index is out of range";                              break;
		case 4003: sErrorDescription = "No memory for function call stack";                        break;
		case 4004: sErrorDescription = "Recursive stack overflow";                                 break;
		case 4005: sErrorDescription = "Not enough stack for parameter";                           break;
		case 4006: sErrorDescription = "No memory for parameter string";                           break;
		case 4007: sErrorDescription = "No memory for temp string";                                break;
		case 4008: sErrorDescription = "Not initialized string";                                   break;
		case 4009: sErrorDescription = "Not initialized string in array";                          break;
		case 4010: sErrorDescription = "No memory for array\' string";                             break;
		case 4011: sErrorDescription = "Too long string";                                          break;
		case 4012: sErrorDescription = "Remainder from zero divide";                               break;
		case 4013: sErrorDescription = "Zero divide";                                              break;
		case 4014: sErrorDescription = "Unknown command";                                          break;
		case 4015: sErrorDescription = "Wrong jump (never generated error)";                       break;
		case 4016: sErrorDescription = "Not initialized array";                                    break;
		case 4017: sErrorDescription = "Dll calls are not allowed";                                break;
		case 4018: sErrorDescription = "Cannot load library";                                      break;
		case 4019: sErrorDescription = "Cannot call function";                                     break;
		case 4020: sErrorDescription = "Expert function calls are not allowed";                    break;
		case 4021: sErrorDescription = "Not enough memory for temp string returned from function"; break;
		case 4022: sErrorDescription = "System is busy (never generated error)";                   break;
		case 4050: sErrorDescription = "Invalid function parameters count";                        break;
		case 4051: sErrorDescription = "Invalid function parameter value";                         break;
		case 4052: sErrorDescription = "String function internal error";                           break;
		case 4053: sErrorDescription = "Some array error";                                         break;
		case 4054: sErrorDescription = "Incorrect series array using";                             break;
		case 4055: sErrorDescription = "Custom indicator error";                                   break;
		case 4056: sErrorDescription = "Arrays are incompatible";                                  break;
		case 4057: sErrorDescription = "Global variables processing error";                        break;
		case 4058: sErrorDescription = "Global variable not found";                                break;
		case 4059: sErrorDescription = "Function is not allowed in testing mode";                  break;
		case 4060: sErrorDescription = "Function is not confirmed";                                break;
		case 4061: sErrorDescription = "Send mail error";                                          break;
		case 4062: sErrorDescription = "String parameter expected";                                break;
		case 4063: sErrorDescription = "Integer parameter expected";                               break;
		case 4064: sErrorDescription = "Double parameter expected";                                break;
		case 4065: sErrorDescription = "Array as parameter expected";                              break;
		case 4066: sErrorDescription = "Requested history data in update state";                   break;
		case 4099: sErrorDescription = "End of file";                                              break;
		case 4100: sErrorDescription = "Some file error";                                          break;
		case 4101: sErrorDescription = "Wrong file name";                                          break;
		case 4102: sErrorDescription = "Too many opened files";                                    break;
		case 4103: sErrorDescription = "Cannot open file";                                         break;
		case 4104: sErrorDescription = "Incompatible access to a file";                            break;
		case 4105: sErrorDescription = "No order selected";                                        break;
		case 4106: sErrorDescription = "Unknown symbol";                                           break;
		case 4107: sErrorDescription = "Invalid price parameter for trade function";               break;
		case 4108: sErrorDescription = "Invalid ticket";                                           break;
		case 4109: sErrorDescription = "Trade is not allowed in the expert properties";            break;
		case 4110: sErrorDescription = "Longs are not allowed in the expert properties";           break;
		case 4111: sErrorDescription = "Shorts are not allowed in the expert properties";          break;
		case 4200: sErrorDescription = "Object is already exist";                                  break;
		case 4201: sErrorDescription = "Unknown object property";                                  break;
		case 4202: sErrorDescription = "Object is not exist";                                      break;
		case 4203: sErrorDescription = "Unknown object type";                                      break;
		case 4204: sErrorDescription = "No object name";                                           break;
		case 4205: sErrorDescription = "Object coordinates error";                                 break;
		case 4206: sErrorDescription = "No specified subwindow";                                   break;
		default:   sErrorDescription = "Unknown error";
	}

	return(sErrorDescription);
}
